"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandRunner = void 0;
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const lodash_padend_1 = __importDefault(require("lodash.padend"));
const lodash_upperfirst_1 = __importDefault(require("lodash.upperfirst"));
const CommandRegistry_1 = require("./CommandRegistry");
class CommandRunner {
    constructor(registry = CommandRegistry_1.CommandRegistry.defaultInstance) {
        this.registry = registry;
        this.program = commander_1.program;
    }
    async run(argv = process.argv) {
        this.program.configureOutput({
            writeErr: (str) => {
                const clear = str.trim().replace(/^error: /i, '');
                this.writeError(lodash_upperfirst_1.default(clear));
            },
        });
        const banner = this.getBanner();
        if (banner) {
            this.program.addHelpText('beforeAll', `${banner}\n`);
        }
        this.registry.getCommands().forEach((cmd) => this.prepareCommand(cmd));
        try {
            await this.program.parseAsync(argv);
        }
        catch (e) {
            this.writeError(e.stack || e);
        }
    }
    useContainer(container) {
        this.container = container;
    }
    setAppBanner(appName, version) {
        this.appBanner = [chalk_1.default.green(appName), version && `(v. ${version})`]
            .filter(Boolean)
            .join(' ');
    }
    setBanner(banner) {
        this.banner = banner;
    }
    onStart(callback) {
        this.onStartCallback = callback;
    }
    onExit(callback) {
        this.onExitCallback = callback;
        this.program.exitOverride(async (error) => {
            await this.onExitCallback(1, error);
        });
    }
    prepareCommand({ target, signature, description, options, }) {
        const cmd = this.program.command(signature);
        if (description) {
            cmd.description(description);
        }
        Object.entries(options || {}).map(([name, opt]) => {
            if (typeof opt === 'string') {
                cmd.addOption(new commander_1.Option(`${opt}, --${name}`));
                return;
            }
            const option = new commander_1.Option([
                opt.short && `-${opt.short},`,
                `--${name}`,
                opt.value === 'required' && `<value${opt.variadic ? '...' : ''}>`,
                opt.value === 'optional' && `[value${opt.variadic ? '...' : ''}]`,
            ]
                .filter(Boolean)
                .join(' '), opt.description);
            if (opt.choices) {
                option.choices(opt.choices);
            }
            if (opt.default) {
                option.default(opt.default, opt.defaultDescription);
            }
            cmd.addOption(option);
        });
        cmd.action(async (...args) => {
            const command = this.container
                ? this.container.get(target)
                : target();
            await this.exec(cmd.name(), command, args.slice(0, args.length - 1));
        });
    }
    async exec(signature, cmd, args) {
        const banner = this.getBanner();
        if (banner) {
            console.log(banner);
            console.log(` ${chalk_1.default.yellow(signature)}`);
            console.log(' ');
        }
        if (this.onStartCallback) {
            this.onStartCallback(signature);
        }
        let exitCode = await cmd.run(...args);
        exitCode = exitCode || 0;
        if (exitCode > 0) {
            console.log('');
            console.log(chalk_1.default.yellow(`Finished with exit code ${exitCode}`));
        }
        if (this.onExitCallback) {
            await this.onExitCallback(exitCode);
        }
    }
    getBanner() {
        return [this.appBanner, this.banner].filter(Boolean).join('\n');
    }
    writeError(str) {
        const lines = str.split('\n');
        const maxLength = Math.max(...lines.map((l) => l.length)) + 2;
        console.log('');
        console.log(chalk_1.default.bgRed.white(lodash_padend_1.default('', maxLength)));
        lines.map((line) => console.log(chalk_1.default.bgRed.white(lodash_padend_1.default(` ${line}`, maxLength))));
        console.log(chalk_1.default.bgRed.white(lodash_padend_1.default('', maxLength)));
        console.log('');
    }
}
exports.CommandRunner = CommandRunner;
CommandRunner.defaultInstance = new CommandRunner();
exports.default = CommandRunner.defaultInstance;
//# sourceMappingURL=CommandRunner.js.map